SystemOrganization addCategory: #'AL-Application-Core'!
SystemOrganization addCategory: #'AL-Application-Utilities'!

Object subclass: #ALApplication
	instanceVariableNames: 'handler locator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALApplication class methodsFor: 'as yet unclassified' stamp: 'cwp 9/27/2011 16:45'!
new
	^ self withLocator: (ALLocator withDigests)! !

!ALApplication class methodsFor: 'as yet unclassified' stamp: 'cwp 3/7/2011 23:08'!
withLocator: aLocator
	^ self basicNew
		initializeWithLocator: aLocator;
		yourself! !

!ALApplication methodsFor: 'api' stamp: 'cwp 9/12/2011 22:50'!
addRelay: aRelay
	aRelay destination: handler.
	handler := aRelay! !

!ALApplication methodsFor: 'error handling' stamp: 'cwp 4/23/2011 18:44'!
handleNotFound: anError
	^ ALResponse notFound: anError messageText! !

!ALApplication methodsFor: 'api' stamp: 'cwp 10/15/2011 00:01'!
handleRequest: aRequest
	^ [ handler handleRequest: aRequest ] 
		on: ALHttpError 
		do:	[:err | 
			aRequest consume.
			err handleWith: self]! !

!ALApplication methodsFor: 'error handling' stamp: 'cwp 10/14/2011 23:53'!
handleUnsupportedContentType: anError 
	^ ALResponse 
		status: 415 
		reason: 'Unsupported Content Type'
		details: anError messageText ! !

!ALApplication methodsFor: 'initialize-release' stamp: 'cwp 9/27/2011 16:20'!
initializeHandler! !

!ALApplication methodsFor: 'initialize-release' stamp: 'cwp 9/27/2011 16:20'!
initializeLocator! !

!ALApplication methodsFor: 'initialize-release' stamp: 'cwp 9/27/2011 16:24'!
initializeWithLocator: aLocator
	locator := aLocator.
	handler := aLocator.
	self initializeLocator.
	self initializeHandler.! !

Object subclass: #ALDirectoryScanner
	instanceVariableNames: 'locator guide ignore root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Utilities'!

!ALDirectoryScanner class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:34'!
locator: aLocator
	^ self basicNew initializeWithLocator: aLocator  ! !

!ALDirectoryScanner class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:46'!
scan: aReference into: aLocator
	(self locator: aLocator) scan: aReference.
	^ aLocator! !

!ALDirectoryScanner methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:37'!
initializeWithLocator: aLocator
	locator := aLocator.
	guide := FSPreorderGuide for: self.! !

!ALDirectoryScanner methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:54'!
scan: aReference
	root := aReference resolve.
	guide show: root! !

!ALDirectoryScanner methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:53'!
visitDirectory: anEntry
	| reference |
	reference := anEntry reference.
	(reference basename first = $.) ifTrue: [ignore := reference].
	! !

!ALDirectoryScanner methodsFor: 'as yet unclassified' stamp: 'cwp 8/17/2011 00:00'!
visitFile: anEntry
	| path reference |
	reference := anEntry reference.
	ignore ifNotNil: [(ignore contains: reference) ifTrue: [^ self]].
	ignore := nil.
	(reference basename first = $.) ifFalse:
		[path := ALPath withAll: (reference relativeTo: root).
		locator at: path put: (ALFileResource on: reference)]! !

Object subclass: #ALLocator
	instanceVariableNames: 'hash map paths resources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALLocator class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 22:25'!
forTesting
	^ self
		hash: [:obj | #[1 2 3]]
		map: [:obj | ALPath / 'obj'].! !

!ALLocator class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 21:41'!
hash: hashBlock map: mapBlock
	^ self basicNew initializeWithHash: hashBlock map: mapBlock ! !

!ALLocator class methodsFor: 'as yet unclassified' stamp: 'cwp 8/20/2011 22:57'!
withDigests
	^ self 
		hash: [ :r | r serialize digest ] 
		map: [ :r | ALPath with: r serialize digest ] ! !

!ALLocator methodsFor: 'public' stamp: 'cwp 8/16/2011 22:54'!
at: aPath alias: aResource
	"Register a resource to be available from a non-canonical path"
	
	resources at: aPath put: aResource.! !

!ALLocator methodsFor: 'public' stamp: 'cwp 8/16/2011 22:37'!
at: aPath put: aResource
	resources at: aPath put: aResource.
	paths at: (hash value: aResource) put: aPath
! !

!ALLocator methodsFor: 'public' stamp: 'cwp 9/12/2011 22:44'!
handleRequest: aRequest
	| path |
	path := aRequest path.
	^ (self resourceForPath: path) handleRequest: aRequest locator: self! !

!ALLocator methodsFor: 'initialize-release' stamp: 'cwp 8/16/2011 21:56'!
initializeWithHash: hashBlock map: mapBlock
	hash := hashBlock.
	map := mapBlock.
	paths := Dictionary new.
	resources := Dictionary new.! !

!ALLocator methodsFor: 'public' stamp: 'cwp 8/16/2011 22:18'!
pathForResource: aResource
	^ paths
		at: (hash value: aResource)
		ifAbsentPut: 
			[| path |
			path := map value: aResource.
			resources at: path put: aResource.
			path]  ! !

!ALLocator methodsFor: 'public' stamp: 'cwp 8/16/2011 22:31'!
resourceForPath: aPath
	^ resources 
		at: aPath 
		ifAbsent: [ALNotFound signal: aPath printString]
		
			! !

!ALLocator methodsFor: 'serializing' stamp: 'cwp 8/6/2011 19:13'!
serializeWith: aSerializer
	aSerializer serializeExternal: self  ! !

Object variableSubclass: #ALPath
	instanceVariableNames: ''
	classVariableNames: 'Slash'
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/20/2011 22:57'!
/ aString
	^ self with: aString! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:16'!
initialize
	Slash := '/'! !

!ALPath class methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 13:45'!
parseWith: aParser
	^ aParser parsePath! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:31'!
root
	^ self new: 0! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:48'!
with: aString
	^ (self new: 1)
		at: 1 put: aString;
		yourself! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:58'!
withAll: segments
	| path |
	path := self new: segments size.
	1 to: path size do: [:index | path at: index put: (segments at: index)].
	^ path! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 9/13/2011 10:49'!
/ aString
	| path |
	(self isRoot and: [aString isEmpty]) ifTrue: [^ self].
	path := self class new: self size + 1.
	1 to: self size do: [:i | path at: i put: (self at: i)].
	path at: (path size) put: aString.
	^ path! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/21/2011 22:20'!
= other
	self species = other species ifFalse: [^ false].
	self size = other size ifFalse: [^ false].
	1 to: self size do: [:i | (self at: i) = (other at: i) ifFalse: [^ false]].
	^ true! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 15:17'!
first
	^ self at: 1! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/21/2011 22:22'!
hash
	| hash |
	hash := self class hash.
	1 to: self size do: [:i | hash := hash bitXor: (self at: i) hash].
	^ hash! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:54'!
isRoot
	^ self size = 0! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 8/21/2011 22:16'!
printOn: aStream
	| ws |
	ws := String new writing.
	self streamingPrintOn: ws.  
	aStream nextPutAll: ws conclusion.! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 15:18'!
rest
	| rest |
	rest := self class new: self size - 1.
	1 to: rest size do: [:index | rest at: index put: (self at: index + 1)].
	^ rest! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:15'!
streamingPrintOn: aWriteStream
	self isRoot 
		ifTrue: [aWriteStream write: Slash]
		ifFalse:
			[1 to: self size do:
				[:index |
				aWriteStream write: Slash.
				aWriteStream write: (self at: index)]]! !

Object subclass: #ALResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

ALResource subclass: #ALCallbackResource
	instanceVariableNames: 'resource callback'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALCallbackResource class methodsFor: 'as yet unclassified' stamp: 'cwp 8/5/2011 22:19'!
resource: aResource callback: aValuable
	^ self basicNew initializeWithResource: aResource callback: aValuable! !

!ALCallbackResource methodsFor: 'as yet unclassified' stamp: 'cwp 8/21/2011 22:30'!
applyRequest: aRequest
	^ self value! !

!ALCallbackResource methodsFor: 'public' stamp: 'cwp 10/1/2011 16:28'!
handleGet: aRequest locator: aLocator
	| path |
	path := aLocator pathForResource: (self applyRequest: aRequest).
	^ ALResponse seeOther: path! !

!ALCallbackResource methodsFor: 'as yet unclassified' stamp: 'cwp 8/5/2011 22:20'!
initializeWithResource: aResource callback: aValuable
	self initialize.
	resource := aResource.
	callback := aValuable.! !

!ALCallbackResource methodsFor: 'as yet unclassified' stamp: 'cwp 8/21/2011 22:30'!
value
	callback value.
	^ resource! !

ALResource subclass: #ALFileResource
	instanceVariableNames: 'reference'
	classVariableNames: 'MimeTypes'
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALFileResource class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:04'!
initialize
	MimeTypes := Dictionary new
		at: 'css' put: ALContentType css;
		at: 'html' put: ALContentType html;
		at: 'ico' put: ALContentType favicon;
		at: 'js' put: ALContentType javascript;
		at: 'less' put: ALContentType less;
		at: 'png' put: ALContentType png;
		yourself.! !

!ALFileResource class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 23:17'!
on: aReference
	^ self new initializeWithReference: aReference! !

!ALFileResource methodsFor: 'comparing' stamp: 'cwp 3/11/2011 23:18'!
= other 
	^ self species = other species and: [self reference = other reference]! !

!ALFileResource methodsFor: 'public' stamp: 'cwp 8/22/2011 22:19'!
contentType
	^ MimeTypes 
		at: reference extension
		ifAbsent: ['application/octet-stream']! !

!ALFileResource methodsFor: 'public' stamp: 'cwp 10/18/2011 16:56'!
handleGet: aRequest locator: aLocator
	| body |
	reference exists ifFalse: [ALNotFound signal].
	body := ALEntity callback: 
		[ :out || in | 
		in := reference reading.
		[out write: reference reading]
			ensure: [in close]].
	body contentType: self contentType.
	body contentLength: reference resolve entry size.
	^ ALResponse status: 200 reason: 'OK' body: body! !

!ALFileResource methodsFor: 'comparing' stamp: 'cwp 3/11/2011 23:18'!
hash
	^ reference hash! !

!ALFileResource methodsFor: 'initialize-release' stamp: 'cwp 3/11/2011 23:18'!
initializeWithReference: aReference
	reference := aReference ! !

!ALFileResource methodsFor: 'accessing' stamp: 'cwp 3/11/2011 23:18'!
reference
	^ reference! !

ALResource subclass: #ALFormResource
	instanceVariableNames: 'resource form'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALFormResource class methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2011 00:29'!
resource: aResource form: aForm
	^ self basicNew
		initializeWithResource: aResource form: aForm ! !

!ALFormResource methodsFor: 'as yet unclassified' stamp: 'cwp 9/24/2011 15:46'!
applyRequest: aRequest
	| data |
	data := aRequest readEntityWith: 
		[ :in | ALFormDataParser parse: (in encoding: #ascii) ].
	form appyData: data.
	^ resource! !

!ALFormResource methodsFor: 'public' stamp: 'cwp 10/1/2011 16:29'!
handle: aRequest locator: aLocator
	| path |
	path := aLocator pathForResource: (self applyRequest: aRequest).
	^ ALResponse seeOther: path! !

!ALFormResource methodsFor: 'public' stamp: 'cwp 10/1/2011 16:29'!
handleGet: aRequest locator: aLocator
	^ self handle: aRequest locator: aLocator! !

!ALFormResource methodsFor: 'public' stamp: 'cwp 10/1/2011 16:29'!
handlePost: aRequest locator: aLocator
	^ self handle: aRequest locator: aLocator! !

!ALFormResource methodsFor: 'as yet unclassified' stamp: 'cwp 9/20/2011 00:28'!
initializeWithResource: aResource form: aForm
	self initialize.
	resource := aResource.
	form := aForm.! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleConnect: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleDelete: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleGet: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleHead: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleOptions: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/6/2011 21:22'!
handlePost: aRequest locator: aLocator
	aRequest readEntityWith: [:in | in rest].  
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/6/2011 21:22'!
handlePut: aRequest locator: aLocator
	aRequest readEntityWith: [:in | in rest].  
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'public' stamp: 'cwp 10/1/2011 16:24'!
handleRequest: aRequest locator: aLocator
	aRequest isMethodConnect ifTrue: [^ self handleConnect: aRequest locator: aLocator].
	aRequest isMethodDelete ifTrue: [^ self handleDelete: aRequest locator: aLocator].
	aRequest isMethodGet ifTrue: [^ self handleGet: aRequest locator: aLocator].
	aRequest isMethodHead ifTrue: [^ self handleHead: aLocator locator: aLocator].
	aRequest isMethodOptions ifTrue: [self handleOptions: aRequest locator: aLocator].
	aRequest isMethodPost ifTrue: [^ self handlePost: aRequest locator: aLocator].
	aRequest isMethodPut ifTrue: [^ self handlePut: aRequest locator: aLocator].
	aRequest isMethodTrace ifTrue: [^ self handleTrace: aRequest locator: aLocator].
	self methodNotAllowed: aRequest method.! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleTrace: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:42'!
methodNotAllowed: aRequest
	^ ALResponse
		status: 405
		reason: 'Method not allowed'
		details: 'The method ', aRequest method, ' is not allowed for ', aRequest path asString! !
ALPath initialize!
ALFileResource initialize!
